"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ErrorRepository = void 0;
var forms_1 = require("@angular/forms");
/**
 * This repository stores and allows usage of generic error messages.
 *
 * Example usage in a component template:
 * ```Html
 * <mat-error *ngFor="let errorCode of errRepo.getErrors('firstName')">
 *     {{ errRepo.getMessage('firstName', errorCode) }}
 * </mat-error>
 * ```
 *
 * If at some point, a custom message is required, you can overwrite the
 * `errorMessages` in this class or blacklist the error code
 * using the `getErrors` method.
 * ```Html
 * <mat-error *ngFor="let errorCode of errRepo.getErrors('firstName', ['required'])">
 *     {{ errRepo.getMessage('firstName', errorCode) }}
 * </mat-error>
 * <mat-error *ngIf="errRepo.hasError('passwordRepeat', 'required')">
 *     This field is of utmost importance.
 * </mat-error>
 * ```
 */
var ErrorRepository = /** @class */ (function () {
    /**
     * @param {FormGroup} form The form used to find fields in.
     */
    function ErrorRepository(form) {
        this.form = form;
    }
    ErrorRepository.addErrors = function (errors) {
        var controls = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            controls[_i - 1] = arguments[_i];
        }
        controls.forEach(function (control) {
            if (control.errors === null) {
                control.setErrors(errors);
            }
            else {
                control.setErrors(__assign(__assign({}, control.errors), errors));
            }
        });
    };
    /**
     * Get all the errors of a certain control. Errors without message generator are ignored.
     * @param {AbstractControl} control The control of which the errors will be returned.
     * @param {Array<string>} blacklist An array of error codes that will be ignored. Use this if
     * you want to write a custom error message for specific error codes.
     */
    ErrorRepository.getErrors = function (control, blacklist) {
        if (blacklist === void 0) { blacklist = []; }
        if (control.errors === null) {
            return [];
        }
        return Object.entries(control.errors)
            .filter(function (_a) {
            var errorCode = _a[0];
            return !blacklist.includes(errorCode);
        })
            .filter(function (_a) {
            var errorCode = _a[0];
            return errorCode in ErrorRepository.errorMessages;
        })
            .map(function (_a) {
            var errorCode = _a[0], error = _a[1];
            return ({
                code: errorCode,
                error: error,
                message: ErrorRepository.errorMessages[errorCode]({
                    control: control,
                    error: error,
                    value: control.value,
                }, ErrorRepository.provided),
            });
        });
    };
    ErrorRepository.removeErrors = function (errorNames) {
        var controls = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            controls[_i - 1] = arguments[_i];
        }
        controls.forEach(function (control) {
            var errors = control.errors;
            if (errors === null) {
                return;
            }
            errorNames.forEach(function (name) { return delete errors[name]; });
            control.setErrors(errors);
        });
    };
    ErrorRepository.getAllControls = function (controls) {
        var _this = this;
        var output = [];
        controls.forEach(function (control) {
            output.push(control);
            if (control instanceof forms_1.FormGroup || control instanceof forms_1.UntypedFormGroup) {
                output.push.apply(output, _this.getAllControls(Object.values(control.controls)));
            }
            else if (control instanceof forms_1.FormArray || control instanceof forms_1.UntypedFormArray) {
                output.push.apply(output, _this.getAllControls(control.controls));
            }
        });
        return output;
    };
    ErrorRepository.prototype.getAllControls = function () {
        return ErrorRepository.getAllControls(Object.values(this.form.controls));
    };
    /**
     * Get all errors of a field.
     * @param path The name of the field to search for or the field itself.
     * @param {Array<string>} blacklist An array of error codes that won't be
     * returned. Use this if you want to write a custom error message for
     * specific error codes.
     */
    ErrorRepository.prototype.getErrors = function (path, blacklist) {
        var field = this.getControl(path);
        if (field === null) {
            console.warn('Field is null, returning empty array list for path', path);
            return [];
        }
        if (!this.isErrorState(field)) {
            return [];
        }
        return ErrorRepository.getErrors(field, blacklist);
    };
    /**
     * Checks whether a certain field has a certain error.
     * @param path The name of the field to search for or the field itself.
     * @param {string} errorCode The error code to search for.
     * @returns {boolean}
     */
    ErrorRepository.prototype.hasError = function (path, errorCode) {
        var control = this.getControl(path);
        if (control === null) {
            console.warn("Control is null, returning no error for code ".concat(errorCode, " path"), path);
            return false;
        }
        var errors = control.errors === null ? [] : Object.keys(control.errors);
        return errors.includes(errorCode);
    };
    /**
     * Checks whether a certain field has any errors.
     * @param path The name of the field to search for or the field itself.
     * @returns {boolean}
     */
    ErrorRepository.prototype.hasErrors = function (path) {
        var control = this.getControl(path);
        if (control === null) {
            console.warn("Control is null, returning no error for path", path);
            return false;
        }
        var errors = control.errors === null ? [] : Object.keys(control.errors);
        return errors.length > 0;
    };
    /**
     * Error when invalid control is dirty, touched, or submitted.
     */
    ErrorRepository.prototype.isErrorState = function (control) {
        if (control === null) {
            return false;
        }
        return control.invalid && (control.dirty || control.touched);
    };
    ErrorRepository.prototype.getControl = function (path) {
        return path === null || path instanceof forms_1.AbstractControl
            ? path
            : this.form.get(path);
    };
    ErrorRepository.errorMessages = {};
    return ErrorRepository;
}());
exports.ErrorRepository = ErrorRepository;
//# sourceMappingURL=index.js.map