import { AbstractControl, FormGroup, ValidationErrors } from '@angular/forms';
export interface ErrorInterface {
    error: any;
    code: string;
    message: string;
}
export interface ErrorMessageInput<Error = any> {
    control: AbstractControl;
    error: Error;
    value: string | null;
}
export declare type ErrorMessage<Error = any, Provided = any> = (err: ErrorMessageInput<Error>, provided: Provided) => string;
export declare type PathType = Array<string | number> | string;
/**
 * This repository stores and allows usage of generic error messages.
 *
 * Example usage in a component template:
 * ```Html
 * <mat-error *ngFor="let errorCode of errRepo.getErrors('firstName')">
 *     {{ errRepo.getMessage('firstName', errorCode) }}
 * </mat-error>
 * ```
 *
 * If at some point, a custom message is required, you can overwrite the
 * `errorMessages` in this class or blacklist the error code
 * using the `getErrors` method.
 * ```Html
 * <mat-error *ngFor="let errorCode of errRepo.getErrors('firstName', ['required'])">
 *     {{ errRepo.getMessage('firstName', errorCode) }}
 * </mat-error>
 * <mat-error *ngIf="errRepo.hasError('passwordRepeat', 'required')">
 *     This field is of utmost importance.
 * </mat-error>
 * ```
 */
export declare class ErrorRepository {
    private readonly form;
    static readonly errorMessages: {
        [key: string]: ErrorMessage;
    };
    /**
     * This property will be provided to all error messages. Use it to e.g. pass a translator
     * instance.
     */
    static provided: any;
    /**
     * @param {FormGroup} form The form used to find fields in.
     */
    constructor(form: FormGroup);
    static addErrors(errors: ValidationErrors, ...controls: Array<AbstractControl>): void;
    /**
     * Get all the errors of a certain control. Errors without message generator are ignored.
     * @param {AbstractControl} control The control of which the errors will be returned.
     * @param {Array<string>} blacklist An array of error codes that will be ignored. Use this if
     * you want to write a custom error message for specific error codes.
     */
    static getErrors(control: AbstractControl, blacklist?: Array<string>): Array<ErrorInterface>;
    static removeErrors(errorNames: Array<string>, ...controls: Array<AbstractControl>): void;
    static getAllControls(controls: Array<AbstractControl>): Array<AbstractControl>;
    getAllControls(): Array<AbstractControl>;
    /**
     * Get all errors of a field.
     * @param path The name of the field to search for or the field itself.
     * @param {Array<string>} blacklist An array of error codes that won't be
     * returned. Use this if you want to write a custom error message for
     * specific error codes.
     */
    getErrors(path: AbstractControl | PathType, blacklist?: Array<string>): Array<ErrorInterface>;
    /**
     * Checks whether a certain field has a certain error.
     * @param path The name of the field to search for or the field itself.
     * @param {string} errorCode The error code to search for.
     * @returns {boolean}
     */
    hasError(path: AbstractControl | PathType, errorCode: string): boolean;
    /**
     * Checks whether a certain field has any errors.
     * @param path The name of the field to search for or the field itself.
     * @returns {boolean}
     */
    hasErrors(path: AbstractControl | PathType): boolean;
    /**
     * Error when invalid control is dirty, touched, or submitted.
     */
    isErrorState(control: AbstractControl | null): boolean;
    private getControl;
}
